@@ -18,6 +18,7 @@ void compute_sizes_from_layout(unsigned monitored_dev_count, unsigned device_hea
                                process_field_displayed process_field_displayed,
                                struct window_position *device_positions, unsigned *num_plots,
                                struct window_position plot_positions[MAX_CHARTS], unsigned *map_device_to_plot,
-                               struct window_position *process_position, struct window_position *setup_position);
+                               struct window_position *process_position, struct window_position *setup_position,
+                               bool process_win_hide);
 
 #endif // INTERFACE_LAYOUT_SELECTION_H__
