@@ -29,7 +29,6 @@
 #include "nvtop/time.h"
 
 #include <assert.h>
-#include <ctype.h>
 #include <dirent.h>
 #include <dlfcn.h>
 #include <errno.h>
@@ -37,7 +36,6 @@
 #include <inttypes.h>
 #include <libdrm/amdgpu.h>
 #include <libdrm/amdgpu_drm.h>
-#include <linux/kcmp.h>
 #include <math.h>
 #include <stdarg.h>
 #include <stdbool.h>
@@ -46,7 +44,6 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/syscall.h>
-#include <sys/sysmacros.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <uthash.h>
@@ -69,6 +66,7 @@ static typeof(drmDropMaster) *_drmDropMaster;
 static typeof(amdgpu_device_initialize) *_amdgpu_device_initialize;
 static typeof(amdgpu_device_deinitialize) *_amdgpu_device_deinitialize;
 static typeof(amdgpu_get_marketing_name) *_amdgpu_get_marketing_name;
+static typeof(amdgpu_query_hw_ip_info) *_amdgpu_query_hw_ip_info;
 static typeof(amdgpu_query_gpu_info) *_amdgpu_query_gpu_info;
 static typeof(amdgpu_query_info) *_amdgpu_query_info;
 static typeof(amdgpu_query_sensor_info) *_amdgpu_query_sensor_info;
@@ -219,6 +217,7 @@ static bool gpuinfo_amdgpu_init(void) {
     _amdgpu_device_initialize = dlsym(libdrm_amdgpu_handle, "amdgpu_device_initialize");
     _amdgpu_device_deinitialize = dlsym(libdrm_amdgpu_handle, "amdgpu_device_deinitialize");
     _amdgpu_get_marketing_name = dlsym(libdrm_amdgpu_handle, "amdgpu_get_marketing_name");
+    _amdgpu_query_hw_ip_info = dlsym(libdrm_amdgpu_handle, "amdgpu_query_hw_ip_info");
     _amdgpu_query_info = dlsym(libdrm_amdgpu_handle, "amdgpu_query_info");
     _amdgpu_query_gpu_info = dlsym(libdrm_amdgpu_handle, "amdgpu_query_gpu_info");
     _amdgpu_query_sensor_info = dlsym(libdrm_amdgpu_handle, "amdgpu_query_sensor_info");
@@ -324,45 +323,53 @@ static void initDeviceSysfsPaths(struct gpu_info_amdgpu *gpu_info) {
   nvtop_device_new_from_syspath(&gpu_info->amdgpuDevice, devicePath);
   assert(gpu_info->amdgpuDevice != NULL);
 
-  int sysfsFD = open(devicePath, O_RDONLY);
   gpu_info->hwmonDevice = nvtop_device_get_hwmon(gpu_info->amdgpuDevice);
-  assert(gpu_info->hwmonDevice != NULL);
-  // Open the device hwmon folder (Fan speed are available there)
-  const char *hwmonPath;
-  nvtop_device_get_syspath(gpu_info->hwmonDevice, &hwmonPath);
-  int hwmonFD = open(hwmonPath, O_RDONLY);
+  if (gpu_info->hwmonDevice) {
+    // Open the device hwmon folder (Fan speed are available there)
+    const char *hwmonPath;
+    nvtop_device_get_syspath(gpu_info->hwmonDevice, &hwmonPath);
+    int hwmonFD = open(hwmonPath, O_RDONLY);
 
-  // Look for which fan to use (PWM or RPM)
-  gpu_info->fanSpeedFILE = NULL;
-  unsigned pwmIsEnabled;
-  int NreadPatterns = readAttributeFromDevice(gpu_info->hwmonDevice, "pwm1_enable", "%u", &pwmIsEnabled);
-  bool usePWMSensor = NreadPatterns == 1 && pwmIsEnabled > 0;
+    // Look for which fan to use (PWM or RPM)
+    gpu_info->fanSpeedFILE = NULL;
+    unsigned pwmIsEnabled;
+    int NreadPatterns = readAttributeFromDevice(gpu_info->hwmonDevice, "pwm1_enable", "%u", &pwmIsEnabled);
+    bool usePWMSensor = NreadPatterns == 1 && pwmIsEnabled > 0;
 
-  bool useRPMSensor = false;
-  if (!usePWMSensor) {
-    unsigned rpmIsEnabled;
-    NreadPatterns = readAttributeFromDevice(gpu_info->hwmonDevice, "fan1_enable", "%u", &rpmIsEnabled);
-    useRPMSensor = NreadPatterns && rpmIsEnabled > 0;
-  }
-  // Either RPM or PWM or neither
-  assert((useRPMSensor ^ usePWMSensor) || (!useRPMSensor && !usePWMSensor));
-  if (usePWMSensor || useRPMSensor) {
-    char *maxFanSpeedFile = usePWMSensor ? "pwm1_max" : "fan1_max";
-    char *fanSensorFile = usePWMSensor ? "pwm1" : "fan1_input";
-    unsigned maxSpeedVal;
-    NreadPatterns = readAttributeFromDevice(gpu_info->hwmonDevice, maxFanSpeedFile, "%u", &maxSpeedVal);
-    if (NreadPatterns == 1) {
-      gpu_info->maxFanValue = maxSpeedVal;
-      // Open the fan file for dynamic info gathering
-      int fanSpeedFD = openat(hwmonFD, fanSensorFile, O_RDONLY);
-      if (fanSpeedFD >= 0) {
-        gpu_info->fanSpeedFILE = fdopen(fanSpeedFD, "r");
-        if (!gpu_info->fanSpeedFILE)
-          close(fanSpeedFD);
+    bool useRPMSensor = false;
+    if (!usePWMSensor) {
+      unsigned rpmIsEnabled;
+      NreadPatterns = readAttributeFromDevice(gpu_info->hwmonDevice, "fan1_enable", "%u", &rpmIsEnabled);
+      useRPMSensor = NreadPatterns && rpmIsEnabled > 0;
+    }
+    // Either RPM or PWM or neither
+    assert((useRPMSensor ^ usePWMSensor) || (!useRPMSensor && !usePWMSensor));
+    if (usePWMSensor || useRPMSensor) {
+      char *maxFanSpeedFile = usePWMSensor ? "pwm1_max" : "fan1_max";
+      char *fanSensorFile = usePWMSensor ? "pwm1" : "fan1_input";
+      unsigned maxSpeedVal;
+      NreadPatterns = readAttributeFromDevice(gpu_info->hwmonDevice, maxFanSpeedFile, "%u", &maxSpeedVal);
+      if (NreadPatterns == 1) {
+        gpu_info->maxFanValue = maxSpeedVal;
+        // Open the fan file for dynamic info gathering
+        int fanSpeedFD = openat(hwmonFD, fanSensorFile, O_RDONLY);
+        if (fanSpeedFD >= 0) {
+          gpu_info->fanSpeedFILE = fdopen(fanSpeedFD, "r");
+          if (!gpu_info->fanSpeedFILE)
+            close(fanSpeedFD);
+        }
       }
     }
+    // Open the power cap file for dynamic info gathering
+    gpu_info->powerCap = NULL;
+    int powerCapFD = openat(hwmonFD, "power1_cap", O_RDONLY);
+    if (powerCapFD) {
+      gpu_info->powerCap = fdopen(powerCapFD, "r");
+    }
+    close(hwmonFD);
   }
 
+  int sysfsFD = open(devicePath, O_RDONLY);
   // Open the PCIe bandwidth file for dynamic info gathering
   gpu_info->PCIeBW = NULL;
   int pcieBWFD = openat(sysfsFD, "pcie_bw", O_RDONLY);
@@ -370,13 +377,6 @@ static void initDeviceSysfsPaths(struct gpu_info_amdgpu *gpu_info) {
     gpu_info->PCIeBW = fdopen(pcieBWFD, "r");
   }
 
-  // Open the power cap file for dynamic info gathering
-  gpu_info->powerCap = NULL;
-  int powerCapFD = openat(hwmonFD, "power1_cap", O_RDONLY);
-  if (powerCapFD) {
-    gpu_info->powerCap = fdopen(powerCapFD, "r");
-  }
-  close(hwmonFD);
   close(sysfsFD);
 }
 
@@ -497,8 +497,10 @@ static int readAttributeFromDevice(nvtop_device *dev, const char *sysAttr, const
   va_start(args, format);
   const char *val;
   int ret = nvtop_device_get_sysattr_value(dev, sysAttr, &val);
-  if (ret < 0)
+  if (ret < 0) {
+    va_end(args);
     return ret;
+  }
   // Read the pattern
   int nread = vsscanf(val, format, args);
   va_end(args);
@@ -513,6 +515,7 @@ static void gpuinfo_amdgpu_populate_static_info(struct gpu_info *_gpu_info) {
   const char *name = NULL;
 
   static_info->integrated_graphics = false;
+  static_info->encode_decode_shared = false;
   RESET_ALL(static_info->valid);
 
   if (libdrm_amdgpu_handle && _amdgpu_get_marketing_name)
@@ -611,17 +614,19 @@ static void gpuinfo_amdgpu_populate_static_info(struct gpu_info *_gpu_info) {
 
   // Critical temparature
   // temp1_* files should always be the GPU die in millidegrees Celsius
-  unsigned criticalTemp;
-  int NreadPatterns = readAttributeFromDevice(gpu_info->hwmonDevice, "temp1_crit", "%u", &criticalTemp);
-  if (NreadPatterns == 1) {
-    SET_GPUINFO_STATIC(static_info, temperature_slowdown_threshold, criticalTemp);
-  }
+  if (gpu_info->hwmonDevice) {
+    unsigned criticalTemp;
+    int NreadPatterns = readAttributeFromDevice(gpu_info->hwmonDevice, "temp1_crit", "%u", &criticalTemp);
+    if (NreadPatterns == 1) {
+      SET_GPUINFO_STATIC(static_info, temperature_slowdown_threshold, criticalTemp);
+    }
 
-  // Emergency/shutdown temparature
-  unsigned emergemcyTemp;
-  NreadPatterns = readAttributeFromDevice(gpu_info->hwmonDevice, "temp1_emergency", "%u", &emergemcyTemp);
-  if (NreadPatterns == 1) {
-    SET_GPUINFO_STATIC(static_info, temperature_shutdown_threshold, emergemcyTemp);
+    // Emergency/shutdown temparature
+    unsigned emergemcyTemp;
+    NreadPatterns = readAttributeFromDevice(gpu_info->hwmonDevice, "temp1_emergency", "%u", &emergemcyTemp);
+    if (NreadPatterns == 1) {
+      SET_GPUINFO_STATIC(static_info, temperature_shutdown_threshold, emergemcyTemp);
+    }
   }
 
   nvtop_pcie_link max_link_characteristics;
@@ -636,6 +641,14 @@ static void gpuinfo_amdgpu_populate_static_info(struct gpu_info *_gpu_info) {
   if (info_query_success && (info.ids_flags & AMDGPU_IDS_FLAGS_FUSION)) {
     static_info->integrated_graphics = true;
   }
+
+  // Checking if Encode and Decode are unified:AMDGPU_INFO_HW_IP_INFO
+  if (_amdgpu_query_hw_ip_info) {
+    struct drm_amdgpu_info_hw_ip vcn_ip_info;
+    if (_amdgpu_query_hw_ip_info(gpu_info->amdgpu_device, AMDGPU_HW_IP_VCN_ENC, 0, &vcn_ip_info) == 0) {
+      static_info->encode_decode_shared = vcn_ip_info.hw_ip_version_major >= 4;
+    }
+  }
 }
 
 static void gpuinfo_amdgpu_refresh_dynamic_info(struct gpu_info *_gpu_info) {
@@ -646,7 +659,6 @@ static void gpuinfo_amdgpu_refresh_dynamic_info(struct gpu_info *_gpu_info) {
   uint32_t out32;
 
   RESET_ALL(dynamic_info->valid);
-  dynamic_info->encode_decode_shared = false;
 
   if (libdrm_amdgpu_handle && _amdgpu_query_gpu_info)
     info_query_success = !_amdgpu_query_gpu_info(gpu_info->amdgpu_device, &info);
@@ -702,7 +714,7 @@ static void gpuinfo_amdgpu_refresh_dynamic_info(struct gpu_info *_gpu_info) {
     // TODO: Determine if we want to include GTT (GPU accessible system memory)
     SET_GPUINFO_DYNAMIC(dynamic_info, total_memory, memory_info.vram.total_heap_size);
     SET_GPUINFO_DYNAMIC(dynamic_info, used_memory, memory_info.vram.heap_usage);
-    SET_GPUINFO_DYNAMIC(dynamic_info, free_memory, memory_info.vram.usable_heap_size - dynamic_info->used_memory);
+    SET_GPUINFO_DYNAMIC(dynamic_info, free_memory, memory_info.vram.total_heap_size - dynamic_info->used_memory);
     SET_GPUINFO_DYNAMIC(dynamic_info, mem_util_rate,
                         (dynamic_info->total_memory - dynamic_info->free_memory) * 100 / dynamic_info->total_memory);
   }
@@ -786,6 +798,7 @@ static const char drm_amdgpu_enc[] = "drm-engine-enc";
 
 static bool parse_drm_fdinfo_amd(struct gpu_info *info, FILE *fdinfo_file, struct gpu_process *process_info) {
   struct gpu_info_amdgpu *gpu_info = container_of(info, struct gpu_info_amdgpu, base);
+  struct gpuinfo_static_info *static_info = &gpu_info->base.static_info;
   static char *line = NULL;
   static size_t line_buf_size = 0;
   ssize_t count = 0;
@@ -814,9 +827,6 @@ static bool parse_drm_fdinfo_amd(struct gpu_info *info, FILE *fdinfo_file, struc
       // Client id is a unique identifier. From the DRM documentation "Unique value relating to the open DRM
       // file descriptor used to distinguish duplicated and shared file descriptors. Conceptually the value should map
       // 1:1 to the in kernel representation of struct drm_file instances."
-      // This information is available for the AMDGPU driver shipping with
-      // the kernel >= 5.19. We still have to use the kcmp syscall to
-      // distinguish duplicated file descriptors for older kernels.
       char *endptr;
       cid = strtoul(val, &endptr, 10);
       if (*endptr)
@@ -959,6 +969,12 @@ static bool parse_drm_fdinfo_amd(struct gpu_info *info, FILE *fdinfo_file, struc
       cache_entry->client_id.pdev = gpu_info->base.pdev;
     }
 
+    // The UI only shows the decode usage when `encode_decode_shared` is true
+    // but amdgpu should only use the encode usage field when it is shared.
+    // Lets add both together for good measure.
+    if (static_info->encode_decode_shared)
+      SET_GPUINFO_PROCESS(process_info, decode_usage, process_info->decode_usage + process_info->encode_usage);
+
 #ifndef NDEBUG
     // We should only process one fdinfo entry per client id per update
     struct amdgpu_process_info_cache *cache_entry_check;
