@@ -26,10 +26,15 @@
 #include <ctype.h>
 #include <dirent.h>
 #include <fcntl.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/syscall.h>
+#ifdef __linux__
+// for major() on linux (available in sys/types.h on BSD)
 #include <sys/sysmacros.h>
+#endif
+#include <sys/types.h>
 #include <unistd.h>
 
 #ifndef KCMP_FILE
