@@ -21,7 +21,6 @@
 
 #include "nvtop/interface.h"
 #include "nvtop/common.h"
-#include "nvtop/extract_gpuinfo.h"
 #include "nvtop/extract_gpuinfo_common.h"
 #include "nvtop/interface_common.h"
 #include "nvtop/interface_internal_common.h"
@@ -36,6 +35,7 @@
 #include <inttypes.h>
 #include <ncurses.h>
 #include <signal.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -43,8 +43,8 @@
 #include <unistd.h>
 
 static unsigned int sizeof_device_field[device_field_count] = {
-    [device_name] = 11,  [device_fan_speed] = 8, [device_temperature] = 10,
-    [device_power] = 15, [device_clock] = 11,    [device_pcie] = 46,
+    [device_name] = 11,       [device_fan_speed] = 11,  [device_temperature] = 10,
+    [device_power] = 15,      [device_clock] = 11,      [device_pcie] = 46,
     [device_shadercores] = 7, [device_l2features] = 11, [device_execengines] = 11,
 };
 
@@ -141,6 +141,9 @@ static void alloc_device_window(unsigned int start_row, unsigned int start_col,
   dwin->decode_util = newwin(1, size_decode, start_row + 2, start_col + spacer * 3 + size_gpu + size_mem + size_encode);
   if (dwin->decode_util == NULL)
     goto alloc_error;
+  dwin->encdec_util = newwin(1, size_encode * 2, start_row + 2, start_col + spacer * 2 + size_gpu + size_mem);
+  if (dwin->encdec_util == NULL)
+    goto alloc_error;
   // For auto-hide encode / decode window
   dwin->gpu_util_no_enc_or_dec = newwin(1, size_gpu + size_encode / 2 + 1, start_row + 2, start_col);
   if (dwin->gpu_util_no_enc_or_dec == NULL)
@@ -160,18 +163,16 @@ static void alloc_device_window(unsigned int start_row, unsigned int start_col,
   dwin->dec_was_visible = false;
 
   // Line 4 = Number of shading cores | L2 Features
-  dwin->shader_cores =
-    newwin(1, sizeof_device_field[device_shadercores], start_row + 3, start_col);
+  dwin->shader_cores = newwin(1, sizeof_device_field[device_shadercores], start_row + 3, start_col);
   if (dwin->shader_cores == NULL)
     goto alloc_error;
-  dwin->l2_cache_size =
-    newwin(1, sizeof_device_field[device_l2features], start_row + 3, start_col + spacer +
-           sizeof_device_field[device_shadercores]);
+  dwin->l2_cache_size = newwin(1, sizeof_device_field[device_l2features], start_row + 3,
+                               start_col + spacer + sizeof_device_field[device_shadercores]);
   if (dwin->l2_cache_size == NULL)
     goto alloc_error;
   dwin->exec_engines =
-    newwin(1, sizeof_device_field[device_execengines], start_row + 3, start_col + spacer * 2 +
-           sizeof_device_field[device_shadercores] + sizeof_device_field[device_l2features]);
+      newwin(1, sizeof_device_field[device_execengines], start_row + 3,
+             start_col + spacer * 2 + sizeof_device_field[device_shadercores] + sizeof_device_field[device_l2features]);
   if (dwin->exec_engines == NULL)
     goto alloc_error;
 
@@ -192,6 +193,7 @@ static void free_device_windows(struct device_window *dwin) {
   delwin(dwin->mem_util_no_enc_and_dec);
   delwin(dwin->encode_util);
   delwin(dwin->decode_util);
+  delwin(dwin->encdec_util);
   delwin(dwin->gpu_clock_info);
   delwin(dwin->mem_clock_info);
   delwin(dwin->power_info);
@@ -359,9 +361,9 @@ static void initialize_all_windows(struct nvtop_interface *dwin) {
   struct window_position setup_position;
 
   compute_sizes_from_layout(devices_count, dwin->options.has_gpu_info_bar ? 4 : 3, device_length(), rows - 1, cols,
-                            dwin->options.gpu_specific_opts, dwin->options.process_fields_displayed,
-                            device_positions, &dwin->num_plots, plot_positions,
-                            map_device_to_plot, &process_position, &setup_position);
+                            dwin->options.gpu_specific_opts, dwin->options.process_fields_displayed, device_positions,
+                            &dwin->num_plots, plot_positions, map_device_to_plot, &process_position, &setup_position,
+                            dwin->options.hide_processes_list);
 
   alloc_plot_window(devices_count, plot_positions, map_device_to_plot, dwin);
 
@@ -641,16 +643,18 @@ static void draw_devices(struct list_head *devices, struct nvtop_interface *inte
     encode_decode_show_select(dev, GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, encoder_rate),
                               GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, decoder_rate),
                               device->dynamic_info.encoder_rate, device->dynamic_info.decoder_rate,
-                              interface->options.encode_decode_hiding_timer, device->dynamic_info.encode_decode_shared,
+                              interface->options.encode_decode_hiding_timer, device->static_info.encode_decode_shared,
                               &display_encode, &display_decode);
 
     WINDOW *gpu_util_win;
     WINDOW *mem_util_win;
     WINDOW *encode_win = dev->encode_util;
     WINDOW *decode_win = dev->decode_util;
-    if (display_encode && display_decode) {
+    if ((display_encode && display_decode) || (display_decode && device->static_info.encode_decode_shared)) {
       gpu_util_win = dev->gpu_util_enc_dec;
       mem_util_win = dev->mem_util_enc_dec;
+      if (device->static_info.encode_decode_shared)
+        decode_win = dev->encdec_util;
     } else {
       if (display_encode || display_decode) {
         // If encode only, place at decode location
@@ -673,7 +677,7 @@ static void draw_devices(struct list_head *devices, struct nvtop_interface *inte
       unsigned rate =
           GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, decoder_rate) ? device->dynamic_info.decoder_rate : 0;
       snprintf(buff, 1024, "%u%%", rate);
-      if (device->dynamic_info.encode_decode_shared)
+      if (device->static_info.encode_decode_shared)
         draw_percentage_meter(decode_win, "ENC/DEC", rate, buff);
       else
         draw_percentage_meter(decode_win, "DEC", rate, buff);
@@ -699,6 +703,15 @@ static void draw_devices(struct list_head *devices, struct nvtop_interface *inte
       snprintf(buff, 1024, "%.3f%s/%.3f%s", used_prefixed, memory_prefix[prefix_off], total_prefixed,
                memory_prefix[prefix_off]);
       draw_percentage_meter(mem_util_win, "MEM", (unsigned int)(100. * used_mem / total_mem), buff);
+    } else if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, total_memory)) {
+      double total_mem = device->dynamic_info.total_memory;
+      double total_prefixed = total_mem;
+      size_t prefix_off;
+      for (prefix_off = 0; prefix_off < 5 && total_prefixed >= 1000.; ++prefix_off) {
+        total_prefixed /= 1024.;
+      }
+      snprintf(buff, 1024, "N/A/%.3f%s", total_prefixed, memory_prefix[prefix_off]);
+      draw_percentage_meter(mem_util_win, "MEM", 0, buff);
     } else {
       snprintf(buff, 1024, "N/A");
       draw_percentage_meter(mem_util_win, "MEM", 0, buff);
@@ -721,14 +734,21 @@ static void draw_devices(struct list_head *devices, struct nvtop_interface *inte
     }
 
     // FAN
-    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, fan_speed))
-      mvwprintw(dev->fan_speed, 0, 0, "FAN %3u%%", device->dynamic_info.fan_speed);
-    else if (device->static_info.integrated_graphics) {
-      mvwprintw(dev->fan_speed, 0, 0, "CPU-FAN ");
-      mvwchgat(dev->fan_speed, 0, 3, 5, 0, cyan_color, NULL);
-    } else
-      mvwprintw(dev->fan_speed, 0, 0, "FAN N/A%%");
-    mvwchgat(dev->fan_speed, 0, 0, 3, 0, cyan_color, NULL);
+    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, fan_speed)) {
+      mvwprintw(dev->fan_speed, 0, 0, " FAN %3u%%  ",
+                device->dynamic_info.fan_speed > 100 ? 100 : device->dynamic_info.fan_speed);
+      mvwchgat(dev->fan_speed, 0, 1, 3, 0, cyan_color, NULL);
+    } else if (device->static_info.integrated_graphics) {
+      mvwprintw(dev->fan_speed, 0, 0, "  CPU-FAN  ");
+      mvwchgat(dev->fan_speed, 0, 2, 7, 0, cyan_color, NULL);
+    } else if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, fan_rpm)) {
+      mvwprintw(dev->fan_speed, 0, 0, "FAN %4uRPM",
+                device->dynamic_info.fan_rpm > 9999 ? 9999 : device->dynamic_info.fan_rpm);
+      mvwchgat(dev->fan_speed, 0, 0, 3, 0, cyan_color, NULL);
+    } else {
+      mvwprintw(dev->fan_speed, 0, 0, "  FAN N/A  ");
+      mvwchgat(dev->fan_speed, 0, 2, 3, 0, cyan_color, NULL);
+    }
     wnoutrefresh(dev->fan_speed);
 
     // GPU CLOCK
@@ -849,7 +869,7 @@ typedef struct {
   struct gpuid_and_process {
     unsigned gpu_id;
     struct gpu_process *process;
-  } * processes;
+  } *processes;
 } all_processes;
 
 static all_processes all_processes_array(struct list_head *devices) {
@@ -1246,24 +1266,30 @@ static void print_processes_on_screen(all_processes all_procs, struct process_wi
       unsigned gpu_usage = 0;
       if (GPUINFO_PROCESS_FIELD_VALID(processes[i].process, gpu_usage)) {
         gpu_usage = processes[i].process->gpu_usage;
+        printed += snprintf(&process_print_buffer[printed], process_buffer_line_size - printed, "%3u%% ", gpu_usage);
+      } else {
+        printed += snprintf(&process_print_buffer[printed], process_buffer_line_size - printed, "N/A  ");
       }
-      printed += snprintf(&process_print_buffer[printed], process_buffer_line_size - printed, "%3u%% ", gpu_usage);
     }
 
     if (process_is_field_displayed(process_enc_rate, fields_to_display)) {
       unsigned encoder_rate = 0;
       if (GPUINFO_PROCESS_FIELD_VALID(processes[i].process, encode_usage)) {
         encoder_rate = processes[i].process->encode_usage;
+        printed += snprintf(&process_print_buffer[printed], process_buffer_line_size - printed, "%3u%% ", encoder_rate);
+      } else {
+        printed += snprintf(&process_print_buffer[printed], process_buffer_line_size - printed, "N/A  ");
       }
-      printed += snprintf(&process_print_buffer[printed], process_buffer_line_size - printed, "%3u%% ", encoder_rate);
     }
 
     if (process_is_field_displayed(process_dec_rate, fields_to_display)) {
       unsigned decode_rate = 0;
       if (GPUINFO_PROCESS_FIELD_VALID(processes[i].process, decode_usage)) {
         decode_rate = processes[i].process->decode_usage;
+        printed += snprintf(&process_print_buffer[printed], process_buffer_line_size - printed, "%3u%% ", decode_rate);
+      } else {
+        printed += snprintf(&process_print_buffer[printed], process_buffer_line_size - printed, "N/A  ");
       }
-      printed += snprintf(&process_print_buffer[printed], process_buffer_line_size - printed, "%3u%% ", decode_rate);
     }
 
     if (process_is_field_displayed(process_memory, fields_to_display)) {
@@ -1277,7 +1303,7 @@ static void print_processes_on_screen(all_processes all_procs, struct process_wi
                    (unsigned)(processes[i].process->gpu_memory_usage / 1048576));
         }
       } else {
-        memory[0] = '\0';
+        snprintf(memory, sizeof_process_field[process_memory], "%s", "N/A");
       }
       printed += snprintf(&process_print_buffer[printed], process_buffer_line_size - printed, "%*s ",
                           sizeof_process_field[process_memory], memory);
@@ -1350,6 +1376,9 @@ static void print_processes_on_screen(all_processes all_procs, struct process_wi
 static void update_process_option_win(struct nvtop_interface *interface);
 
 static void draw_processes(struct list_head *devices, struct nvtop_interface *interface) {
+  if (interface->options.hide_processes_list)
+    return;
+
   if (interface->process.process_win == NULL)
     return;
 
@@ -1553,6 +1582,10 @@ static void draw_process_shortcuts(struct nvtop_interface *interface) {
   switch (current_state) {
   case nvtop_option_state_hidden:
     for (size_t i = 0; i < ARRAY_SIZE(option_selection_hidden); ++i) {
+      if (interface->options.hide_processes_list &&
+          (strcmp(option_selection_hidden_num[i], "6") == 0 || strcmp(option_selection_hidden_num[i], "9") == 0))
+        continue;
+
       if (process_field_displayed_count(interface->options.process_fields_displayed) > 0 || (i != 1 && i != 2)) {
         wprintw(win, "F%s", option_selection_hidden_num[i]);
         wattr_set(win, A_STANDOUT, cyan_color, NULL);
@@ -2022,3 +2055,93 @@ bool show_information_messages(unsigned num_messages, const char **messages) {
   }
   return dontShowAgainOption;
 }
+
+void print_snapshot(struct list_head *devices, bool use_fahrenheit_option) {
+  gpuinfo_populate_static_infos(devices);
+  gpuinfo_refresh_dynamic_info(devices);
+  struct gpu_info *device;
+
+  printf("[\n");
+  list_for_each_entry(device, devices, list) {
+    const char *indent_level_two = "  ";
+    const char *indent_level_four = "   ";
+
+    const char *device_name_field = "device_name";
+    const char *gpu_clock_field = "gpu_clock";
+    const char *mem_clock_field = "mem_clock";
+    const char *temp_field = "temp";
+    const char *fan_field = "fan_speed";
+    const char *power_field = "power_draw";
+    const char *gpu_util_field = "gpu_util";
+    const char *mem_util_field = "mem_util";
+
+    printf("%s{\n", indent_level_two);
+
+    // Device Name
+    if (GPUINFO_STATIC_FIELD_VALID(&device->static_info, device_name))
+      printf("%s\"%s\": \"%s\",\n", indent_level_four, device_name_field, device->static_info.device_name);
+    else
+      printf("%s\"%s\": null,\n", indent_level_four, device_name_field);
+
+    // GPU Clock Speed
+    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, gpu_clock_speed))
+      printf("%s\"%s\": \"%uMHz\",\n", indent_level_four, gpu_clock_field, device->dynamic_info.gpu_clock_speed);
+    else
+      printf("%s\"%s\": null,\n", indent_level_four, gpu_clock_field);
+
+    // MEM Clock Speed
+    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, mem_clock_speed))
+      printf("%s\"%s\": \"%uMHz\",\n", indent_level_four, mem_clock_field, device->dynamic_info.mem_clock_speed);
+    else
+      printf("%s\"%s\": null,\n", indent_level_four, mem_clock_field);
+
+    // GPU Temperature
+    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, gpu_temp)) {
+      unsigned int temp_convert;
+      if (!use_fahrenheit_option)
+        temp_convert = device->dynamic_info.gpu_temp;
+      else
+        temp_convert = (unsigned)(32 + nearbyint(device->dynamic_info.gpu_temp * 1.8));
+
+      printf("%s\"%s\": \"%u%s\",\n", indent_level_four, temp_field, temp_convert, use_fahrenheit_option ? "F" : "C");
+    } else {
+      printf("%s\"%s\": null,\n", indent_level_four, temp_field);
+    }
+
+    // Fan speed
+    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, fan_speed))
+      printf("%s\"%s\": \"%u%%\",\n", indent_level_four, fan_field,
+             device->dynamic_info.fan_speed > 100 ? 100 : device->dynamic_info.fan_speed);
+    else if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, fan_rpm))
+      printf("%s\"%s\": \"%uRPM\",\n", indent_level_four, fan_field,
+             device->dynamic_info.fan_rpm > 9999 ? 9999 : device->dynamic_info.fan_rpm);
+    else if (device->static_info.integrated_graphics)
+      printf("%s\"%s\": \"CPU Fan\",\n", indent_level_four, fan_field);
+    else
+      printf("%s\"%s\": null,\n", indent_level_four, fan_field);
+
+    // Power draw
+    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, power_draw))
+      printf("%s\"%s\": \"%uW\",\n", indent_level_four, power_field, device->dynamic_info.power_draw / 1000);
+    else
+      printf("%s\"%s\": null,\n", indent_level_four, power_field);
+
+    // GPU Utilization
+    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, gpu_util_rate))
+      printf("%s\"%s\": \"%u%%\",\n", indent_level_four, gpu_util_field, device->dynamic_info.gpu_util_rate);
+    else
+      printf("%s\"%s\": null,\n", indent_level_four, gpu_util_field);
+
+    // Memory Utilization
+    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, mem_util_rate))
+      printf("%s\"%s\": \"%u%%\"\n", indent_level_four, mem_util_field, device->dynamic_info.mem_util_rate);
+    else
+      printf("%s\"%s\": null\n", indent_level_four, mem_util_field);
+
+    if (device->list.next == devices)
+      printf("%s}\n", indent_level_two);
+    else
+      printf("%s},\n", indent_level_two);
+  }
+  printf("]\n");
+}
