@@ -19,16 +19,6 @@
  *
  */
 
-#include <getopt.h>
-#include <ncurses.h>
-#include <signal.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-
-#include <locale.h>
-
 #include "nvtop/extract_gpuinfo.h"
 #include "nvtop/info_messages.h"
 #include "nvtop/interface.h"
@@ -37,6 +27,17 @@
 #include "nvtop/time.h"
 #include "nvtop/version.h"
 
+#include <getopt.h>
+#include <ncurses.h>
+#include <signal.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+
+#include <locale.h>
+
 static volatile sig_atomic_t signal_exit = 0;
 static volatile sig_atomic_t signal_resize_win = 0;
 
@@ -56,6 +57,7 @@ static const char helpstring[] = "Available options:\n"
                                  "  -c --config-file  : Provide a custom config file location to load/save "
                                  "preferences\n"
                                  "  -p --no-plot      : Disable bar plot\n"
+                                 "  -P --no-processes : Disable process list\n"
                                  "  -r --reverse-abs  : Reverse abscissa: plot the recent data left and "
                                  "older on the right\n"
                                  "  -C --no-color     : No colors\n"
@@ -64,7 +66,9 @@ static const char helpstring[] = "Available options:\n"
                                  "  -i --gpu-info     : Show bar with additional GPU parametres\n"
                                  "  -E --encode-hide  : Set encode/decode auto hide time in seconds "
                                  "(default 30s, negative = always on screen)\n"
-                                 "  -h --help         : Print help and exit\n";
+                                 "  -h --help         : Print help and exit\n"
+                                 "  -s --snapshot     : Output the current gpu stats without ncurses"
+                                 "(useful for scripting)\n";
 
 static const char versionString[] = "nvtop version " NVTOP_VERSION_STRING;
 
@@ -79,11 +83,13 @@ static const struct option long_opts[] = {
     {.name = "gpu-info", .has_arg = no_argument, .flag = NULL, .val = 'i'},
     {.name = "encode-hide", .has_arg = required_argument, .flag = NULL, .val = 'E'},
     {.name = "no-plot", .has_arg = no_argument, .flag = NULL, .val = 'p'},
+    {.name = "no-processes", .has_arg = no_argument, .flag = NULL, .val = 'P'},
     {.name = "reverse-abs", .has_arg = no_argument, .flag = NULL, .val = 'r'},
+    {.name = "snapshot", .has_arg = no_argument, .flag = NULL, .val = 's'},
     {0, 0, 0, 0},
 };
 
-static const char opts[] = "hvd:c:CfE:pri";
+static const char opts[] = "hvd:c:CfE:pPris";
 
 int main(int argc, char **argv) {
   (void)setlocale(LC_CTYPE, "");
@@ -94,9 +100,11 @@ int main(int argc, char **argv) {
   bool no_color_option = false;
   bool use_fahrenheit_option = false;
   bool hide_plot_option = false;
+  bool hide_processes_option = false;
   bool reverse_plot_direction_option = false;
   bool encode_decode_timer_option_set = false;
   bool show_gpu_info_bar = false;
+  bool show_snapshot = false;
   double encode_decode_hide_time = -1.;
   char *custom_config_file_path = NULL;
   while (true) {
@@ -151,9 +159,15 @@ int main(int argc, char **argv) {
     case 'p':
       hide_plot_option = true;
       break;
+    case 'P':
+      hide_processes_option = true;
+      break;
     case 'r':
       reverse_plot_direction_option = true;
       break;
+    case 's':
+      show_snapshot = true;
+      break;
     case ':':
     case '?':
       switch (optopt) {
@@ -201,6 +215,12 @@ int main(int argc, char **argv) {
     return EXIT_SUCCESS;
   }
 
+  if (show_snapshot) {
+    print_snapshot(&monitoredGpus, use_fahrenheit_option);
+    gpuinfo_shutdown_info_extraction(&monitoredGpus);
+    return EXIT_SUCCESS;
+  }
+
   unsigned numWarningMessages = 0;
   const char **warningMessages;
   get_info_messages(&monitoredGpus, &numWarningMessages, &warningMessages);
@@ -230,6 +250,7 @@ int main(int argc, char **argv) {
       allDevicesOptions.gpu_specific_opts[i].to_draw = 0;
     }
   }
+  allDevicesOptions.hide_processes_list = hide_processes_option;
   if (encode_decode_timer_option_set) {
     allDevicesOptions.encode_decode_hiding_timer = encode_decode_hide_time;
     if (allDevicesOptions.encode_decode_hiding_timer < 0.)
